import java.io.* ;
import java.util.* ;

class Freq {

  private static int nwords = 0 ;

  static void count(WordReader in, Assoc t) {
    String word ;
    while ((word = in.read()) != null) {
      if (word.length() >= 4) {
	nwords++ ;
        word = word.toLowerCase() ;
        t.put(word, t.get(word)+1) ;
      }
    }
  }

  private static boolean test = false ;
  private static long time = 0 ;
  private static long ptime = 0 ;
  private static long interval = 500000000 ;

  static void run(WordReader in, Assoc t) {
    long tstart = System.nanoTime() ;
    count(in, t) ;
    long tstop = System.nanoTime() ;
    time += tstop-tstart ;
    if (test && time - ptime > interval) {
      ptime = time ;
      double ftime = time/1000000000.0 ;
      String msg = nwords + " " + ftime ;
      if (test) System.out.println(msg) ;
    }
  }

  static void run(String name, Assoc t) {
    try {
      WordReader in = new WordReader (new FileReader(name)) ;
      run(in, t) ;
      in.close() ;
    } catch (IOException e) {
      System.err.println(e.getMessage()) ;
    }
  }

  static void run(Iterator<String> names, Assoc t) {
    while (names.hasNext()) {
      run(names.next(), t) ;
    }
  }
  
  private static void usage() {
    System.err.println("Usage: java Freq [options]* [files]*") ;
    System.err.println("Print the words and the number of their occurences") ;
    System.err.println("  Argument [files]* are filenames or files of filenames when of the form @name") ;
    System.err.println("  When no argument is present, input is read on standard input") ;
    System.err.println("Simple example: java Freq *.java") ;
    System.err.println() ;
    System.err.println("Options are:") ;
    System.err.println("  -t, test mode, print runtime statistics") ;
    System.err.println("  -i n, test mode, print runtime statistics every 0.n seconds") ;
    System.err.println("  -help, print this message") ;
    System.err.println("Other options command the implementation of association tables.") ;
    System.err.println("  -Lib library hashtables (HashMap)") ;
    System.err.println("  -H custom hashtables, chaining") ;
    System.err.println("  -H2 custom hashtables, chaining, alternative hash function") ;
    System.err.println("  -O custom hashtables, linear probing") ;
    System.err.println("  -D custom hashtables, double hashing") ;
    System.err.println("  -L association list") ;
    System.err.println("  -C association list with caching of one search") ;
    System.err.println("  -T library trees (TreeMap)") ;
    System.err.println("  -S custom trees") ;
      
  }

  public static void main(String [] arg) {
    Assoc t = null ;
    int start = 0 ;
    int len = arg.length ;
    while (start < len) {
      String a = arg[start] ;
      if (a.equals("-Lib")) {
	t = new Lib() ;
	start++ ;
      } else if (a.equals("-H")) {
	t = new H() ;
	start++ ;
      } else if (a.equals("-H2")) {
	t = new H2() ;
	start++ ;
      } else if (a.equals("-O")) {
	t = new O() ;
	start++ ;
      } else if (a.equals("-D")) {
	t = new D() ;
	start++ ;
      } else if (a.equals("-L")) {
	t = new L() ;
	start++ ;
      } else if (a.equals("-C")) {
	t = new C() ;
	start++ ;
      } else if (a.equals("-T")) {
	t = new T() ;
	start++ ;
      } else if (a.equals("-S")) {
	t = new S() ;
	start++ ;
      } else if (a.equals("-t")) {
	test = true ;
	start++ ;
      } else if (a.equals("-i")) {
	test = true ;
	start++ ;
	interval = 100000000 * Integer.parseInt(arg[start]) ;
	start++ ;
      } else if (a.equals("-help")) {
        usage() ;
        System.exit(2) ;
      } else {
	break ;
      }
    }
    if (t == null) t = new H () ;

    if (len-start == 0) {
      count(new WordReader (new InputStreamReader (System.in)), t) ;
      System.out.println(t) ;
    } else {
      for (int k = start ; k < len ; k++) {
	String name = arg[k] ;
	if (name.length() > 0 && name.charAt(0) == '@') {
	  try {
	    run(new Scanner (new FileReader (name.substring(1))), t) ;
	  } catch (FileNotFoundException e) {
	    System.err.println(e.getMessage()) ;
	  }
	} else
	  run(name, t) ;
      }
      if (!test) System.out.print(t) ;
    }
  }
}
